#  Copyright 2023 Fabarta Authors.
# 
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import json
import toml

DEFAULT_CONFIG_PATH = './tools_config.toml'
CSV_SUFFIX = ".csv"
LDBC_SUFFIX = "_0_0"


class Config:
    def __init__(self, config_path=DEFAULT_CONFIG_PATH):
        print(f"with config file {config_path}")
        self.config_path = config_path
        with open(self.config_path, 'r') as f:
            self.config = toml.load(f)

            # sort
            self.pool_size = self.get_config('performance', 'pool_size')
            self.driver_memory = self.get_config('performance', 'spark_driver_memory')
            self.client_data_dir = self.get_config('import', 'client_data_dir')
            self.sorted_csv_dir = self.get_config('import', 'sorted_csv_dir')
            self.hotspot_dir = self.get_config('import', 'hotspot_dir')
            self.tmp_dir = self.get_config('import', 'tmp_dir')
            self.csv_delimiter = self.get_config('import', 'csv_delimiter')
            self.csv_quote = self.get_config('import', 'csv_quote')
            self.csv_escape = self.get_config('import', 'csv_escape')
            self.csv_has_headers = self.get_config('import', 'csv_has_headers')
            self.csv_name_format = self.get_config('import', 'csv_name_format')
            self.error_dir = self.get_config('import', 'error_dir')
            self.check_duplicate = self.get_config('import', 'check_duplicate')
            self.on_duplicate = self.get_config('import', 'on_duplicate')
            self.reverse_edge = self.get_config('import', 'reverse_edge')
            self.rank_type = json.loads(self.get_config('import', 'rank_type'))
            self.rank_timestamp_format = json.loads(self.get_config('import', 'rank_timestamp_format'))
            self.spark_dir = self.get_config('import', 'spark_dir')
            # upload
            self.server_import_path = self.get_config('import', 'server_import_path')
            self.server_addresses = self.get_config('upload', 'server_addresses')
            self.server_usernames = self.get_config('upload', 'server_usernames')
            self.server_passwords = self.get_config('upload', 'server_passwords')
            

    def get_config(self, section_name, config_name):
        return self.config[section_name][config_name]

    def get_type_file_name(self, type_name):
        if self.csv_name_format == 'ldbc':
            return '{}{}{}'.format(type_name, LDBC_SUFFIX, CSV_SUFFIX)
        else:
            return '{}{}'.format(type_name, CSV_SUFFIX)

    def is_timestamp_rank(self, edge_name):
        return self.rank_type[edge_name] == 'timestamp'
    
    def get_timestamp_rank_fmt(self, edge_name):
        return self.rank_timestamp_format[edge_name]