#  Copyright 2023 Fabarta Authors.
# 
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import json
import os
import time
import sys

from config import Config
from schema import read_schema
from spark_processor import SparkProcessor, EDGE_KEY_DIR, OUT_EDGE_KEY_DIR, IN_EDGE_KEY_DIR



if __name__ == '__main__':
    print('start processing encoded csv files.')
    if len(sys.argv) > 1:
        config = Config(sys.argv[1])
        spark = SparkProcessor(config)
    else:
        config = Config()
        spark = SparkProcessor(config)
    vertex_schema, edge_schema = read_schema(config)

    start = time.time()
    # part 1: vertex indexes
    for (vertex_name, schema) in vertex_schema.items():
        schema = json.loads(schema)    
        if len(schema['indexes']) > 0:
            for index_schema in schema['indexes']:
                index_name = index_schema['index_name']
                index_dir = os.path.join(config.tmp_dir, 'indexes', index_name) # this is a directory
                if index_schema['is_primary_key']:
                    spark.process_pkoid_dir(index_dir, index_name)
                else:
                    spark.process_normal_index_dir(index_dir, index_name)
    # part 2: edge key
    edge_key_dir = os.path.join(config.tmp_dir, EDGE_KEY_DIR)
    out_edge_key_dir, in_edge_key_dir = os.path.join(edge_key_dir, OUT_EDGE_KEY_DIR), os.path.join(edge_key_dir, IN_EDGE_KEY_DIR)
    if os.path.exists(out_edge_key_dir):
        for dir in os.listdir(out_edge_key_dir):
            spark.process_edge_key_dir(os.path.join(out_edge_key_dir, dir), dir, OUT_EDGE_KEY_DIR)
    if os.path.exists(in_edge_key_dir):
        for dir in os.listdir(in_edge_key_dir):
            spark.process_edge_key_dir(os.path.join(in_edge_key_dir, dir), dir, IN_EDGE_KEY_DIR)
    
    end = time.time()
    print("sorting encoded csv in spark costs {:.2f} seconds in total.".format(end - start))
