#  Copyright 2023 Fabarta Authors.
# 
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import json
import time
import sys

from config import Config
from schema import read_schema
from spark_processor import SparkProcessor


if __name__ == "__main__":
    if len(sys.argv) > 1:
        config = Config(sys.argv[1])
        spark = SparkProcessor(config)
    else:
        config = Config()
        spark = SparkProcessor(config)
    vertex_schemas, edge_schemas = read_schema(config)

    start = time.time()
    # process vertex files
    for (vertex_name, vertex_schema) in vertex_schemas.items():
        vertex_schema = json.loads(vertex_schema)
        file_name = config.get_type_file_name(vertex_name)
        spark.process_vertex_file(file_name, vertex_schema)

    # process edge files
    for (edge_full_name, edge_schema) in edge_schemas.items():
        edge_schema = json.loads(edge_schema)
        file_name = config.get_type_file_name(edge_full_name)
        spark.process_edge_file(file_name, edge_schema, edge_full_name, vertex_schemas)

    end = time.time()
    print("sorting csv in spark costs {:.2f} seconds in total.".format(end - start))
