#  Copyright 2023 Fabarta Authors.
# 
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

# spark-submit --driver-class-path ./mysql-connector-java-5.1.49.jar spark_jdbc_processor.py

from pyspark.sql import SparkSession
from pyspark.sql.functions import col

# Initialize a SparkSession
spark = SparkSession.builder.master('local[8]').appName("MySQL to CSV with Limit").getOrCreate()

# MySQL JDBC URL
url = "jdbc:mysql://127.0.0.1:3306/aiplatform"
properties = {
    "user": "root",
    "password": "passw0rd",
    "driver": "com.mysql.jdbc.Driver"
}

# Define batch size for pagination
batch_size = 100
offset = 0

# Assuming we have a DataFrame to accumulate the results
# You can also write each batch to CSV and combine later if the dataset is huge
accumulated_df = None

# Loop to read the data batch by batch
while True:
    # Query with LIMIT and OFFSET
    # Other db maybe different from mysql to page select
    query = "(SELECT * FROM person LIMIT {} OFFSET {}) as batch".format(batch_size, offset)

    # Read the batch into a DataFrame
    batch_df = spark.read.jdbc(url=url, table=query, properties=properties)

    # If the batch is empty, we have reached the end of the table
    if batch_df.count() == 0:
        break

    # If we have a dataframe already, union the new batch to it
    # Otherwise, the batch itself becomes the accumulated_df
    if accumulated_df is not None:
        accumulated_df = accumulated_df.union(batch_df)
    else:
        accumulated_df = batch_df

    # Increase the offset for the next batch
    offset += batch_size

# Once the loop is done, you have read all the data
# Now you can save the accumulated DataFrame as a CSV file
if accumulated_df is not None:
    accumulated_df.write.csv("./person")

# Stop the SparkSession
spark.stop()