#  Copyright 2023 Fabarta Authors.
# 
#  Licensed under the Apache License, Version 2.0 (the "License");
#  you may not use this file except in compliance with the License.
#  You may obtain a copy of the License at
# 
#      http://www.apache.org/licenses/LICENSE-2.0
# 
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS,
#  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#  See the License for the specific language governing permissions and
#  limitations under the License.

import os, sys
import paramiko
import threading

from scp import SCPClient
from config import Config


def createSSHClient(server, port, user, password):
    client = paramiko.SSHClient()
    client.load_system_host_keys()
    client.set_missing_host_key_policy(paramiko.AutoAddPolicy())
    client.connect(server, port, user, password)
    return client


if __name__ == "__main__":
    task_uuid = sys.argv[1]
    if len(sys.argv) > 2:
        config = Config(sys.argv[2])
    else:
        config = Config()
    server_addresses = config.server_addresses.split(';')
    server_usernames = config.server_usernames.split(';')
    server_passwords = config.server_passwords.split(';')

    assert (len(server_addresses) == len(server_usernames) and len(server_usernames) == len(server_passwords))
    num_servers = len(server_addresses)
    threads = list()
    for i in range(num_servers):
        ssh = createSSHClient(server_addresses[i], 22, server_usernames[i], server_passwords[i])
        ssh.exec_command('mkdir -p {}'.format(config.server_import_path))
        scp = SCPClient(ssh.get_transport())
        t = threading.Thread(target=scp.put(os.path.join(config.tmp_dir, 'sst'),
                                            os.path.join(config.server_import_path, task_uuid), True))
        threads.append(t)
        t.start()

    for thread in threads:
        thread.join()
